import UIKit

public protocol TransformDelegate : AnyObject {
    func undoManager() -> UndoManager
    func isUndoEnabled() -> Bool
    func isRedoEnabled() -> Bool
    func isUndoing() -> Bool
    func isRedoing() -> Bool
    func undo()
    func redo()
    func transformState(_ transform: Any)
}

public class TransformRecord: NSObject {
    
    public var actionName : String!

    var useCurrent : Bool! = true

    let previousValues :  Dictionary<String, Any?>!
    var currentValues : Dictionary<String, Any?>!
    
    public init(actionName: String, previousValues: [String : Any?], currentValues: [String : Any?]) {
        
        self.actionName = actionName
        self.previousValues = previousValues
        self.currentValues = currentValues
        
        super.init()
    }

    public required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    public func updateTransformState() {
            let transformState = self.useCurrent ? self.currentValues[.kCurrentTransformState] as! Transformation : self.previousValues[.kCurrentTransformState] as! Transformation
            
        TransformStack.shared.sharedTransformDelegate.transformState(transformState)
    }
    
    // Add/Redo
    @objc public func addAdjustmentToStack(_ applyTransform: NSNumber? = nil) {
        
        self.useCurrent = true
        
        if let applyTransform = applyTransform?.boolValue {
            
            if applyTransform {
                self.updateTransformState()
            }
        }
        
        TransformStack.shared.pushTransformRecord(self)

        // register the undo event
        TransformStack.shared.sharedTransformDelegate.undoManager().registerUndo(withTarget: self, selector: #selector(removeAdjustmentFromStack), object: nil)
        
        TransformStack.shared.sharedTransformDelegate.undoManager().setActionName(self.actionName)
    }
    
    // Undo
     @objc public func removeAdjustmentFromStack() {
        
         self.useCurrent = false
         
         self.updateTransformState()
         
         TransformStack.shared.popTransformStack()
         let applyTransform = true
         TransformStack.shared.sharedTransformDelegate.undoManager().registerUndo(withTarget: self, selector: #selector(addAdjustmentToStack), object: NSNumber(booleanLiteral: applyTransform))
        
         TransformStack.shared.sharedTransformDelegate.undoManager().setActionName(self.actionName)
    }
}

extension String {
    public static let kCurrentTransformState = "CurrentTransformState"
}

